@@ -161,6 +161,16 @@ gail_get_accessible_for_widget (GtkWidget *widget,
 
       obj = gtk_widget_get_accessible (widget);
       focus_object = g_object_get_qdata (G_OBJECT (obj), quark_focus_object);
+      /*
+       * We check whether the object for this focus_object has been deleted.
+       * This can happen when navigating to an empty directory in nautilus. 
+       * See bug #141907.
+       */
+      if (ATK_IS_GOBJECT_ACCESSIBLE (focus_object))
+        {
+          if (!atk_gobject_accessible_get_object (ATK_GOBJECT_ACCESSIBLE (focus_object)))
+            focus_object = NULL;
+        }
       if (focus_object)
         obj = focus_object;
     }
@@ -205,8 +215,9 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
                       if (GTK_IS_MENU_ITEM (next_focus_widget) &&
                           !focus_before_menu)
                         {
+                          void *vp_focus_before_menu = &focus_before_menu;
                           focus_before_menu = window->focus_widget;
-                          g_object_add_weak_pointer (G_OBJECT (focus_before_menu), (gpointer *)&focus_before_menu);
+                          g_object_add_weak_pointer (G_OBJECT (focus_before_menu), vp_focus_before_menu);
                         }
 
                       return TRUE;
@@ -346,8 +357,9 @@ gail_finish_select (GtkWidget *widget)
           (GTK_IS_MENU_BAR (next_focus_widget) ||
            GTK_IS_MENU_ITEM (next_focus_widget)))
         {
+          void *vp_next_focus_widget = &next_focus_widget;
           g_source_remove (focus_notify_handler);
-          g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
+          g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), vp_next_focus_widget);
           focus_notify_handler = 0;
           was_deselect = FALSE;
         }
@@ -360,8 +372,9 @@ gail_finish_select (GtkWidget *widget)
       !GTK_IS_MENU_ITEM (focus_widget) && 
       !GTK_IS_MENU (focus_widget))
     {
+      void *vp_focus_before_menu = &focus_before_menu;
       focus_before_menu = focus_widget;
-      g_object_add_weak_pointer (G_OBJECT (focus_before_menu), (gpointer *)&focus_before_menu);
+      g_object_add_weak_pointer (G_OBJECT (focus_before_menu), vp_focus_before_menu);
 
     } 
   gail_focus_notify_when_idle (widget);
@@ -476,7 +489,8 @@ gail_focus_idle_handler (gpointer data)
     }
   else
     {
-      g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
+      void *vp_next_focus_widget = &next_focus_widget;
+      g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), vp_next_focus_widget);
       next_focus_widget = NULL;
     }
     
@@ -495,19 +509,22 @@ gail_focus_notify (GtkWidget *widget)
     {
       if (focus_widget)
         {
-          g_object_remove_weak_pointer (G_OBJECT (focus_widget), (gpointer *)&focus_widget);
+          void *vp_focus_widget = &focus_widget;
+          g_object_remove_weak_pointer (G_OBJECT (focus_widget), vp_focus_widget);
         }
       focus_widget = widget;
       if (focus_widget)
         {
-          g_object_add_weak_pointer (G_OBJECT (focus_widget), (gpointer *)&focus_widget);
+          void *vp_focus_widget = &focus_widget;
+          g_object_add_weak_pointer (G_OBJECT (focus_widget), vp_focus_widget);
           /*
            * The UI may not have been updated yet; e.g. in gtkhtml2
            * html_view_layout() is called in a idle handler
            */
           if (focus_widget == focus_before_menu)
             {
-              g_object_remove_weak_pointer (G_OBJECT (focus_before_menu), (gpointer *)&focus_before_menu);
+              void *vp_focus_before_menu = &focus_before_menu;
+              g_object_remove_weak_pointer (G_OBJECT (focus_before_menu), vp_focus_before_menu);
               focus_before_menu = NULL;
             }
         }
@@ -560,7 +577,10 @@ gail_focus_notify_when_idle (GtkWidget *widget)
             }
           g_source_remove (focus_notify_handler);
           if (next_focus_widget)
-            g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
+	    {
+	      void *vp_next_focus_widget = &next_focus_widget;
+	      g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), vp_next_focus_widget);
+	    }
         }
       else
         /*
@@ -571,8 +591,9 @@ gail_focus_notify_when_idle (GtkWidget *widget)
 
   if (widget)
     {
+      void *vp_next_focus_widget = &next_focus_widget;
       next_focus_widget = widget;
-      g_object_add_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
+      g_object_add_weak_pointer (G_OBJECT (next_focus_widget), vp_next_focus_widget);
     }
   focus_notify_handler = g_idle_add (gail_focus_idle_handler, widget);
 }
@@ -607,8 +628,9 @@ gail_deactivate_watcher (GSignalInvocationHint *ihint,
       (GTK_IS_MENU_BAR (next_focus_widget) ||
        GTK_IS_MENU_ITEM (next_focus_widget)))
     {
+      void *vp_next_focus_widget = &next_focus_widget;
       g_source_remove (focus_notify_handler);
-      g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), (gpointer *)&next_focus_widget);
+      g_object_remove_weak_pointer (G_OBJECT (next_focus_widget), vp_next_focus_widget);
       focus_notify_handler = 0;
       was_deselect = FALSE;
     }
@@ -678,7 +700,11 @@ gail_focus_tracker_init ()
 static void
 gail_focus_object_destroyed (gpointer data)
 {
-  g_object_set_qdata (G_OBJECT (data), quark_focus_object, NULL);
+  GObject *obj;
+
+  obj = G_OBJECT (data);
+  g_object_set_qdata (obj, quark_focus_object, NULL);
+  g_object_unref (obj); 
 }
 
 static void
@@ -711,7 +737,16 @@ gail_focus_tracker (AtkObject *focus_object)
                     g_object_weak_unref (G_OBJECT (old_focus_object),
                                          (GWeakNotify) gail_focus_object_destroyed,
                                          parent);
-                  g_object_weak_ref (G_OBJECT (focus_object), 
+                  else
+                    /*
+                     * We call g_object_ref as if the parent is destroyed 
+                     * while the weak reference exists then destroying the 
+                     * child would cause gail_focus_object_destroyed to be 
+                     * called when parent is not a valid GObject.
+                     */
+                    g_object_ref (parent);
+
+                  g_object_weak_ref (G_OBJECT (focus_object),
                                      (GWeakNotify) gail_focus_object_destroyed,
                                      parent);
                   g_object_set_qdata (G_OBJECT (parent), quark_focus_object, focus_object);
@@ -727,6 +762,7 @@ gail_focus_tracker (AtkObject *focus_object)
                                    (GWeakNotify) gail_focus_object_destroyed,
                                    focus_object);
               g_object_set_qdata (G_OBJECT (focus_object), quark_focus_object, NULL);
+              g_object_unref (G_OBJECT (focus_object));
             }
         }
     }
@@ -781,6 +817,7 @@ gail_accessibility_module_init (void)
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CELL_RENDERER_TEXT, gail_text_cell);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CELL_RENDERER_TOGGLE, gail_boolean_cell);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CELL_RENDERER_PIXBUF, gail_image_cell);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CELL_RENDERER, gail_renderer_cell);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_RADIO_BUTTON, gail_radio_button);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_ARROW, gail_arrow);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_PIXMAP, gail_pixmap);
