@@ -32,6 +32,7 @@ static gint         gail_combo_box_get_n_children          (AtkObject      *obj)
 static AtkObject*   gail_combo_box_ref_child               (AtkObject      *obj,
                                                             gint           i);
 static void         gail_combo_box_finalize                (GObject        *object);
+#if 0
 static void         atk_action_interface_init              (AtkActionIface *iface);
 
 static gboolean     gail_combo_box_do_action               (AtkAction      *action,
@@ -46,6 +47,7 @@ static G_CONST_RETURN gchar* gail_combo_box_action_get_name(AtkAction      *acti
 static gboolean              gail_combo_box_set_description(AtkAction      *action,
                                                             gint           i,
                                                             const gchar    *desc);
+#endif
 
 static void         atk_selection_interface_init           (AtkSelectionIface *iface);
 static gboolean     gail_combo_box_add_selection           (AtkSelection   *selection,
@@ -82,12 +84,14 @@ gail_combo_box_get_type (void)
         NULL /* value table */
       };
 
+#if 0
       static const GInterfaceInfo atk_action_info =
       {
         (GInterfaceInitFunc) atk_action_interface_init,
         (GInterfaceFinalizeFunc) NULL,
         NULL
       };
+#endif
       static const GInterfaceInfo atk_selection_info =
       {
         (GInterfaceInitFunc) atk_selection_interface_init,
@@ -189,7 +193,7 @@ gail_combo_box_changed_gtk (GtkWidget *widget)
   gail_combo_box = GAIL_COMBO_BOX (obj);
   if (gail_combo_box->old_selection != index)
     {
-      gail_combo_box->old_selection == index;
+      gail_combo_box->old_selection = index;
       g_signal_emit_by_name (obj, "selection_changed");
     }
 }
@@ -267,7 +271,6 @@ static AtkObject*
 gail_combo_box_ref_child (AtkObject *obj,
                           gint      i)
 {
-  AtkObject *accessible;
   GtkWidget *widget;
 
   g_return_val_if_fail (GAIL_IS_COMBO_BOX (obj), NULL);
@@ -283,6 +286,7 @@ gail_combo_box_ref_child (AtkObject *obj,
   return NULL;
 }
 
+#if 0
 static void
 atk_action_interface_init (AtkActionIface *iface)
 {
@@ -329,7 +333,6 @@ static gboolean
 idle_do_action (gpointer data)
 {
   GtkComboBox *combo_box;
-  GtkWidget *action_widget;
   GtkWidget *widget;
   GailComboBox *gail_combo_box;
 
@@ -384,6 +387,25 @@ gail_combo_box_action_get_name (AtkAction *action,
     return NULL;
 }
 
+static gboolean
+gail_combo_box_set_description (AtkAction   *action,
+                                gint        i,
+                                const gchar *desc)
+{
+  if (i == 0)
+    {
+      GailComboBox *combo_box;
+
+      combo_box = GAIL_COMBO_BOX (action);
+      g_free (combo_box->press_description);
+      combo_box->press_description = g_strdup (desc);
+      return TRUE;
+    }
+  else
+    return FALSE;
+}
+#endif
+
 static void
 atk_selection_interface_init (AtkSelectionIface *iface)
 {
@@ -445,9 +467,6 @@ gail_combo_box_ref_selection (AtkSelection *selection,
                               gint         i)
 {
   GtkComboBox *combo_box;
-  GList * list;
-  GtkWidget *item;
-  AtkObject *obj;
   GtkWidget *widget;
 
   widget = GTK_ACCESSIBLE (selection)->widget;
@@ -494,7 +513,6 @@ gail_combo_box_is_child_selected (AtkSelection *selection,
                                   gint         i)
 {
   GtkComboBox *combo_box;
-  GtkWidget *item;
   gint j;
   GtkWidget *widget;
 
@@ -522,24 +540,6 @@ gail_combo_box_remove_selection (AtkSelection *selection,
   return TRUE;
 }
 
-static gboolean
-gail_combo_box_set_description (AtkAction   *action,
-                                gint        i,
-                                const gchar *desc)
-{
-  if (i == 0)
-    {
-      GailComboBox *combo_box;
-
-      combo_box = GAIL_COMBO_BOX (action);
-      g_free (combo_box->press_description);
-      combo_box->press_description = g_strdup (desc);
-      return TRUE;
-    }
-  else
-    return FALSE;
-}
-
 static void
 gail_combo_box_finalize (GObject *object)
 {
