@@ -35,7 +35,7 @@ static AtkObject*            gail_expander_ref_child        (AtkObject         *
 static AtkStateSet*          gail_expander_ref_state_set    (AtkObject         *obj);
 static void                  gail_expander_real_notify_gtk  (GObject           *obj,
                                                              GParamSpec        *pspec);
-static gint                  gail_expander_map_gtk          (GtkWidget         *widget,
+static void                  gail_expander_map_gtk          (GtkWidget         *widget,
                                                              gpointer          data);
 
 static void                  gail_expander_real_initialize  (AtkObject         *obj,
@@ -209,10 +209,12 @@ gail_expander_new (GtkWidget *widget)
 static G_CONST_RETURN gchar*
 gail_expander_get_name (AtkObject *obj)
 {
+  G_CONST_RETURN gchar *name;
   g_return_val_if_fail (GAIL_IS_EXPANDER (obj), NULL);
 
-  if (obj->name != NULL)
-    return obj->name;
+  name = ATK_OBJECT_CLASS (parent_class)->get_name (obj);
+  if (name != NULL)
+    return name;
   else
     {
       /*
@@ -267,7 +269,7 @@ gail_expander_real_initialize (AtkObject *obj,
   obj->role = ATK_ROLE_TOGGLE_BUTTON;
 }
 
-static gint
+static void
 gail_expander_map_gtk (GtkWidget *widget,
                        gpointer data)
 {
