@@ -188,6 +188,8 @@ gail_option_menu_real_add_gtk (GtkContainer *container,
   AtkObject* atk_parent = ATK_OBJECT (data);
   AtkObject* atk_child = gtk_widget_get_accessible (widget);
 
+  GAIL_CONTAINER_CLASS (parent_class)->add_gtk (container, widget, data);
+
   g_object_notify (G_OBJECT (atk_child), "accessible_parent");
 
   g_signal_emit_by_name (atk_parent, "children_changed::add",
@@ -208,9 +210,9 @@ gail_option_menu_real_remove_gtk (GtkContainer *container,
   g_value_init (&values.old_value, G_TYPE_POINTER);
   g_value_set_pointer (&values.old_value, atk_parent);
 
-  values.property_name = "accessible_parent";
+  values.property_name = "accessible-parent";
   g_signal_emit_by_name (atk_child,
-                         "property_change::accessible_parent", &values, NULL);
+                         "property_change::accessible-parent", &values, NULL);
   g_signal_emit_by_name (atk_parent, "children_changed::remove",
 			 1, atk_child, NULL);
 
