@@ -382,9 +382,9 @@ menu_item_remove_gtk (GtkContainer *container,
       gail_container = GAIL_CONTAINER (atk_parent);
       g_value_init (&values.old_value, G_TYPE_POINTER);
       g_value_set_pointer (&values.old_value, atk_parent);
-      values.property_name = "accessible_parent";
+      values.property_name = "accessible-parent";
       g_signal_emit_by_name (atk_child,
-                             "property_change::accessible_parent", &values, NULL);
+                             "property_change::accessible-parent", &values, NULL);
 
       index = g_list_index (gail_container->children, widget);
       g_list_free (gail_container->children);
