@@ -510,7 +510,11 @@ gail_widget_ref_state_set (AtkObject *accessible)
   
       if (GTK_WIDGET_HAS_FOCUS (widget))
         {
-          atk_state_set_add_state (state_set, ATK_STATE_FOCUSED);
+          AtkObject *focus_obj;
+
+          focus_obj = g_object_get_data (G_OBJECT (accessible), "gail-focus-object");
+          if (focus_obj == NULL)
+            atk_state_set_add_state (state_set, ATK_STATE_FOCUSED);
         }
     }
   return state_set;
@@ -978,7 +982,12 @@ static void
 gail_widget_focus_event (AtkObject   *obj,
                          gboolean    focus_in)
 {
-  atk_object_notify_state_change (obj, ATK_STATE_FOCUSED, focus_in);
+  AtkObject *focus_obj;
+
+  focus_obj = g_object_get_data (G_OBJECT (obj), "gail-focus-object");
+  if (focus_obj == NULL)
+    focus_obj = obj;
+  atk_object_notify_state_change (focus_obj, ATK_STATE_FOCUSED, focus_in);
 }
 
 static GtkWidget*
