@@ -48,9 +48,6 @@ struct _GailWidgetClass
 {
   GtkAccessibleClass parent_class;
 
-  void (*init)                         (GailWidget          *widget,
-                                        GtkWidget           *gtk_widget);
- 
   /*
    * Signal handler for notify signal on GTK widget
    */
@@ -66,15 +63,6 @@ struct _GailWidgetClass
 
 AtkObject*     gail_widget_new         (GtkWidget       *widget);
 
-void           gail_widget_init        (GailWidget      *widget,
-                                        GtkWidget       *gtk_widget);
-
-void           gail_widget_get_origins (GtkWidget       *widget,
-                                        gint		*x_widget,
-                                        gint		*y_widget,
-                                        gint		*x_window,
-                                        gint		*y_window);
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
